#!/usr/bin/env node
const URL = require("url");
const fetch = require("node-fetch");

const { JSDOM } = require("jsdom");
const program = require("commander");

const SPEC_GENERATOR_URL = "https://labs.w3.org/spec-generator/?type=respec";
const LINKCHECKER_URL = "https://validator.w3.org/checklink?url=";

const colors = require("colors");
colors.setTheme({
  error: "red",
  info: "green",
  details: "blue",
});

program
  .version("0.0.1")
  .usage("--spec URL")
  .option("--spec [URL]", "The URL to your spec");

program.on("spec", async specURL => {
  console.log(colors.info("☕ Link checker can take about 1-2 mins... \n"));
  const linkCheckerComponent = encodeURIComponent(
    `${SPEC_GENERATOR_URL}&url=${specURL}`
  );
  const url =
    LINKCHECKER_URL +
    linkCheckerComponent +
    "&summary=on&hide_type=all&depth=&check=Check";
  const html = await fetch(url).then(r => r.text());
  checkForErrors(html);
});
function checkForErrors(html) {
  const dom = new JSDOM(html);
  const invalidLinks = dom.window.document.querySelectorAll(
    ".report dt[id$='404']"
  );
  if (!invalidLinks.length) {
    console.log(colors.info("👍 is all good!"));
    return process.exit(0);
  }
  Array.from(invalidLinks).map(elem =>
    console.error(
      colors.error("💀 " + elem.textContent + "\n"),
      colors.info(elem.nextElementSibling.textContent + "\n"),
      colors.details(
        elem.nextElementSibling.nextElementSibling.textContent + "\n"
      ),
      "\n"
    )
  );
  process.exit(1);
}

program.parse(process.argv);
